@@ -29,6 +29,9 @@ varimp <- function (object, mincriterion = 0, conditional = FALSE,
 {
 
     response <- object@responses
+    if (length(response@variables) == 1 && 
+        inherits(response@variables[[1]], "Surv"))
+        return(varimpsurv(object, mincriterion, conditional, threshold, nperm, OOB, pre1.0_0))
     input <- object@data@get("input")
     xnames <- colnames(input)
     inp <- initVariableFrame(input, trafo = NULL)
@@ -70,7 +73,7 @@ varimp <- function (object, mincriterion = 0, conditional = FALSE,
 
 
             ## if OOB == TRUE use only oob observations, otherwise use all observations in learning sample
-            if(OOB){oob <- object@weights[[b]] == 0} else{ oob <- rep(TRUE, length(xnames))}
+            if(OOB){oob <- object@weights[[b]] == 0} else{ oob <- rep(TRUE, length(y))}
             p <- .Call("R_predict", tree, inp, mincriterion, -1L, PACKAGE = "party")
             eoob <- error(p, oob)
 
@@ -106,6 +109,87 @@ varimp <- function (object, mincriterion = 0, conditional = FALSE,
 }
 
 
+varimpsurv <- function (object, mincriterion = 0, conditional = FALSE, 
+                        threshold = 0.2, nperm = 1, OOB = TRUE, pre1.0_0 = conditional)
+{
+
+    cat("\n")
+    cat("Variable importance for survival forests; this feature is _experimental_\n\n")
+    response <- object@responses
+    input <- object@data@get("input")
+    xnames <- colnames(input)
+    inp <- initVariableFrame(input, trafo = NULL)
+    y <- object@responses@variables[[1]]
+    weights <- object@initweights
+    stopifnot(inherits(y, "Surv"))
+
+    if (conditional || pre1.0_0) {
+        if(!all(complete.cases(inp@variables)))
+            stop("cannot compute variable importance measure with missing values")
+    }
+    stopifnot(require("ipred", quietly = TRUE))
+    error <- function(x, oob) sbrier(y[oob,,drop = FALSE], x[oob])
+
+    pred <- function(tree, newinp) {
+
+        where <- .Call("R_get_nodeID", tree, inp, mincriterion, PACKAGE = "party")
+        wh <- .Call("R_get_nodeID", tree, newinp, mincriterion, PACKAGE = "party")
+        swh <- sort(unique(wh))
+        RET <- vector(mode = "list", length = length(wh))
+        for (i in 1:length(swh)) {
+            w <- weights * (where == swh[i])
+            RET[wh == swh[i]] <- list(mysurvfit(y, weights = w))
+        }
+        return(RET)
+    }
+
+    w <- object@initweights
+    if (max(abs(w - 1)) > sqrt(.Machine$double.eps))
+        warning(sQuote("varimp"), " with non-unity weights might give misleading results")
+
+    ## list for several permutations
+    perror <- matrix(0, nrow = nperm*length(object@ensemble), ncol = length(xnames))
+    ## this matrix is initialized with values 0 so that a tree that does not 
+    ## contain the current variable adds importance 0 to its average importance
+    colnames(perror) <- xnames
+        for (b in 1:length(object@ensemble)){
+            tree <- object@ensemble[[b]]
+
+
+            ## if OOB == TRUE use only oob observations, otherwise use all observations in learning sample
+            if(OOB){oob <- object@weights[[b]] == 0} else{ oob <- rep(TRUE, length(y))}
+            p <- pred(tree, inp)
+            eoob <- error(p, oob)
+
+            ## for all variables (j = 1 ... number of variables) 
+            for(j in unique(varIDs(tree))){
+              for (per in 1:nperm){
+
+                    tmp <- inp
+                    ccl <- create_cond_list(conditional, threshold, xnames[j], input)
+                    if (is.null(ccl)) {
+                        perm <- sample(which(oob))
+                    } else {
+                        perm <- conditional_perm(ccl, xnames, input, tree, oob)
+                    }
+                    tmp@variables[[j]][which(oob)] <- tmp@variables[[j]][perm]
+                    p <- pred(tree, tmp)
+
+                ## run through all rows of perror
+                perror[(per+(b-1)*nperm), j] <- (error(p, oob) - eoob)
+
+              } ## end of for (per in 1:nperm)
+            } ## end of for(j in unique(varIDs(tree)))
+        } ## end of for (b in 1:length(object@ensemble))
+
+    perror <- as.data.frame(perror)
+    #return(MeanDecreaseAccuracy = perror) ## return the whole matrix (= nperm*ntree values per variable)
+    return(MeanDecreaseAccuracy = colMeans(perror)) ## return only averages over permutations and trees
+}
+
+
+
+
 # cutpoints_list() returns:
 # - vector of cutpoints (length=number of cutpoints) 
 #   if variable is continuous
